from django.contrib import messages
from django.contrib.auth import get_user_model
from django.contrib.auth.mixins import LoginRequiredMixin
from django.core.exceptions import ObjectDoesNotExist
from django.core.signing import BadSignature
from django.http import HttpResponse
from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _
from django.views import View
from django.views.generic import ListView, FormView

from whistle import settings
from whistle.forms import NotificationSettingsForm
from whistle.models import Notification


class NotificationListView(LoginRequiredMixin, ListView):
    model = Notification
    paginate_by = 10

    def dispatch(self, request, *args, **kwargs):
        if request.user.is_authenticated:
            request.user.notifications.unread().mark_as_read()
            request.user.clear_unread_notifications_cache()
        return super(NotificationListView, self).dispatch(request, *args, **kwargs)

    def get_queryset(self):
        return self.request.user.notifications.select_related('actor', 'recipient')


class NotificationSettingsView(LoginRequiredMixin, FormView):
    form_class = NotificationSettingsForm
    success_url = reverse_lazy('notifications:settings')  # TODO: configurable
    template_name = 'whistle/settings.html'

    def form_valid(self, form):
        user = self.request.user
        user.notification_settings = form.cleaned_data
        user.save(update_fields=['notification_settings'])
        messages.success(self.request, _('Notification settings successfully updated'))
        return super(NotificationSettingsView, self).form_valid(form)

    def get_form_kwargs(self):
        form_kwargs = super(NotificationSettingsView, self).get_form_kwargs()
        form_kwargs['user'] = self.request.user
        return form_kwargs


class ReadNotificationByHashView(View):
    def dispatch(self, *args, **kwargs):
        hash = kwargs.get('hash')

        from django.core import signing

        try:
            notification_data = signing.loads(hash, key=settings.SIGNING_KEY, salt=settings.SIGNING_SALT)
        except BadSignature:
            return HttpResponse('BAD SIGNATURE')

        notification_id = notification_data.get('notification_id', None)
        recipient_id = notification_data.get('recipient_id', None)

        try:
            notification = Notification.objects.get(pk=notification_id)

            if notification.is_read:
                return HttpResponse('ALREADY READ')

            user = get_user_model().objects.get(pk=recipient_id)

            if notification.recipient != user:
                return HttpResponse('INVALID RECIPIENT')

            notification.is_read = True
            notification.save(update_fields=['is_read'])
            user.clear_unread_notifications_cache()

        except ObjectDoesNotExist:
            return HttpResponse('NOT FOUND')

        return HttpResponse('OK')



from django.core.exceptions import ObjectDoesNotExist
from django.views.generic import DetailView

from whistle.models import Notification


class ReadNotificationMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        if request.user.is_authenticated:
            # read notification by its id
            notification_id = request.GET.get('read-notification', None)

            if notification_id:
                try:
                    notification = Notification.objects.get(pk=notification_id, is_read=False)

                    if notification.recipient == request.user:
                        notification.is_read = True
                        notification.save(update_fields=['is_read'])
                        request.user.clear_unread_notifications_cache()
                except ObjectDoesNotExist:
                    pass

        # read notifications by context
        response = self.get_response(request)
        reload_response = False

        try:
            context = response.context_data
            view = context.get('view')

            if isinstance(view, DetailView):
                # read notifications by object
                object = context.get('object')

                unread_notifications = Notification.objects\
                    .unread()\
                    .for_recipient(request.user)\
                    .of_object_or_target(object)

                if unread_notifications.exists():
                    unread_notifications.update(is_read=True)
                    request.user.clear_unread_notifications_cache()
                    reload_response = True

        except AttributeError:
            pass

        if reload_response:
            response = self.get_response(request)

        return response



from django_rq import job
from django.core.mail import send_mail
from whistle.managers import NotificationManager
from whistle import settings as whistle_settings


def notify(request, recipient, event, actor=None, object=None, target=None, details=''):
    NotificationManager.notify(request, recipient, event, actor, object, target, details)


@job(whistle_settings.REDIS_QUEUE)
def send_mail_in_background(subject, message, from_email, recipient_list, html_message=None, fail_silently=True):
    send_mail(subject, message, from_email, recipient_list, html_message=html_message, fail_silently=fail_silently)



from django.core.management import call_command
from django.db import migrations


def notification_to_web(apps, schema_editor):
    try:
        call_command('copy_channel_settings', 'notification', 'web', delete=True)
    except:
        pass


def web_to_notification(apps, schema_editor):
    try:
        call_command('copy_channel_settings', 'web', 'notification', delete=True)
    except:
        pass


class Migration(migrations.Migration):

    dependencies = [
        ('whistle', '0002_notification_details'),
    ]

    operations = [
        migrations.RunPython(notification_to_web, web_to_notification),
    ]



import re

from crispy_forms.bootstrap import FormActions
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Div, HTML, Field, Layout, Submit
from django import forms
from django.utils.translation import ugettext_lazy as _, ugettext

from whistle.managers import NotificationManager
from whistle.models import Notification
from whistle import settings as whistle_settings


class NotificationAdminForm(forms.ModelForm):
    class Meta:
        model = Notification
        exclude = ()

    def clean(self):
        event = self.cleaned_data.get('event', None)

        if event is not None:
            event_display = dict(whistle_settings.EVENTS).get(event)
            error = _('This field is required for selected event.')

            ctx_mapping = {
                'object_id': 'object',
                'target_id': 'target',
                'actor': 'actor',
            }

            for field, variable in ctx_mapping.items():
                value = self.cleaned_data.get(field, None)
                if value is None and '%({})s'.format(variable) in event_display:
                    self.add_error(field, error)

        return self.cleaned_data


class NotificationSettingsForm(forms.Form):
    def __init__(self, user, *args, **kwargs):
        super(NotificationSettingsForm, self).__init__(*args, **kwargs)
        self.user = user
        self.init_fields()
        self.init_form_helper()

    @property
    def labels(self):
        events = dict(whistle_settings.EVENTS)

        for event, label in events.items():
            pat = re.compile(r'%\(.*\)s|"%\(.*\)s"|%\(.*\)r|"%\(.*\)r"')
            new_label = re.sub(pat, '', ugettext(label))  # remove all variable placeholders
            new_label = new_label.replace("''", '')  # remove all 2 single quotas
            new_label = new_label.replace('""', '')  # remove all 2 double quotas
            new_label = new_label.strip(' :.')  # remove trailing spaces and semicolons
            new_label = re.sub(' +', ' ', new_label)  # remove all multiple spaces

            events[event] = new_label

        return events

    def field_names(self, event):
        event_identifier = event.lower()

        return {
            'web': 'web_{}'.format(event_identifier),
            'mail': 'mail_{}'.format(event_identifier),
            'push': 'push_{}'.format(event_identifier)
        }

    def get_initial_value(self, channel, event):
        return NotificationManager.is_notification_enabled(self.user, channel, event)

    def init_fields(self):
        for event, label in self.labels.items():
            field_names = self.field_names(event)

            self.fields.update({
                field_names['web']: forms.BooleanField(label=_('Web'), required=False, initial=self.get_initial_value('web', event)),
                field_names['mail']: forms.BooleanField(label=_('Mail'), required=False, initial=self.get_initial_value('mail', event)),
                field_names['push']: forms.BooleanField(label=_('Push'), required=False, initial=self.get_initial_value('push', event))
            })

    def init_form_helper(self):
        fields = []

        for event, label in self.labels.items():
            field_names = self.field_names(event)

            event_fields = [
                Div(Field(field_names['web'], css_class='switch'), css_class='col-md'),
                Div(Field(field_names['mail'], css_class='switch'), css_class='col-md'),
            ]

            if whistle_settings.PUSH_NOTIFICATIONS_ENABLED:
                event_fields.append(
                    Div(Field(field_names['push'], css_class='switch'), css_class='col-md')
                )

            field = Div(
                Div(HTML('<p>{}</p>'.format(label)), css_class='col-md-6'),
                *event_fields,
                css_class='row'
            )

            fields.append(field)

        fields.append(
            FormActions(
                Submit('submit', _('Save'), css_class='btn-lg')
            )
        )

        self.helper = FormHelper()
        self.helper.form_class = 'notification-settings'
        self.helper.layout = Layout(*fields)

    def clean(self):
        settings = {'mail': {}, 'web': {}, 'push': {}}

        for event in self.labels.keys():
            field_names = self.field_names(event)
            event_identifier = event.lower()

            settings['web'][event_identifier] = self.cleaned_data.get(field_names['web'], False)
            settings['mail'][event_identifier] = self.cleaned_data.get(field_names['mail'], False)
            settings['push'][event_identifier] = self.cleaned_data.get(field_names['push'], False)

        return settings



from __future__ import unicode_literals

import json
import re
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.shortcuts import get_current_site
from django.core.exceptions import ObjectDoesNotExist
from django.core.mail import send_mail
from django.db.models import QuerySet, Q
from django.template import loader, TemplateDoesNotExist

from whistle import settings as whistle_settings


class NotificationQuerySet(QuerySet):
    def unread(self):
        return self.filter(is_read=False)

    def mark_as_read(self):
        return self.update(is_read=True)

    def for_recipient(self, recipient):
        return self.filter(recipient=recipient) if recipient.is_authenticated else self.none()

    def of_object(self, object):
        return self.filter(
            object_content_type=ContentType.objects.get_for_model(object),
            object_id=object.id
        )

    def of_target(self, target):
        return self.filter(
            target_content_type=ContentType.objects.get_for_model(target),
            target_id=target.id
        )
    
    def of_object_or_target(self, obj):
        return self.filter(
            Q(object_content_type=ContentType.objects.get_for_model(obj), object_id=obj.id) |
            Q(target_content_type=ContentType.objects.get_for_model(obj), target_id=obj.id)
        )


class NotificationManager(object):
    @staticmethod
    def is_notification_enabled(user, channel, event):
        event_identifier = event.lower()

        notification_settings = user.notification_settings

        # support for django-jsonfield which breaks native PostgreSQL functionality
        if isinstance(notification_settings, str):
            notification_settings = json.loads(notification_settings)

        try:
            return notification_settings[channel][event_identifier]
        except (KeyError, TypeError):
            return True

    @staticmethod
    def notify(request, recipient, event, actor=None, object=None, target=None, details=''):
        if not recipient.is_active:
            return

        from whistle.models import Notification

        # create new notification object
        notification = Notification(
            recipient=recipient,
            event=event,
            actor=actor,
            object=object,
            target=target,
            details=details
        )

        # web
        if NotificationManager.is_notification_enabled(recipient, 'web', event):
            # save notification to DB
            notification.save()

            # clear user notifications cache
            recipient.clear_unread_notifications_cache()

        # email
        if NotificationManager.is_notification_enabled(recipient, 'mail', event):
            notification.send_mail(request)

        # push
        if NotificationManager.is_notification_enabled(recipient, 'push', event):
            notification.push(request)

    @staticmethod
    def get_description(event, actor, object, target, pass_variables=True):
        event_template = dict(whistle_settings.EVENTS).get(event)

        event_context = {
            'actor': actor,
            'object': object,
            'target': target,
        } if pass_variables else {
            'actor': '',
            'object': '',
            'target': '',
        }

        if object:
            object_content_type = ContentType.objects.get_for_model(object)
            event_context[object_content_type.model.lower()] = object if pass_variables else ''

        if target:
            target_content_type = ContentType.objects.get_for_model(target)
            event_context[target_content_type.model.lower()] = target if pass_variables else ''

        description = event_template % event_context

        if not pass_variables:
            description = description.replace("''", '')   # remove all 2 single quotas
            description = description.replace('""', '')   # remove all 2 double quotas
            description = description.strip(' :.')         # remove trailing spaces and semicolons
            description = re.sub(' +', ' ', description)  # remove all multiple spaces
        return description


class EmailManager(object):
    @staticmethod
    def send_mail(request, recipient, event, actor=None, object=None, target=None, details='', hash=None):
        """
        Send email notification about a new event to its recipient
        """

        # template
        try:
            t = loader.get_template('whistle/mails/{}.txt'.format(event.lower()))
        except TemplateDoesNotExist:
            t = loader.get_template('whistle/mails/new_notification.txt'.format(event.lower()))

        # HTML template
        try:
            t_html = loader.get_template('whistle/mails/{}.html'.format(event.lower()))
        except TemplateDoesNotExist:
            try:
                t_html = loader.get_template('whistle/mails/new_notification.html')
            except TemplateDoesNotExist:
                t_html = None

        # recipients
        recipient_list = [recipient.email]

        # description
        description = NotificationManager.get_description(event, actor, object, target, True)

        # subject
        short_description = NotificationManager.get_description(event, actor, object, target, False)

        try:
            site = get_current_site(request)

            subject = '[{}] {}'.format(
                site.name,
                short_description  # TODO: add setting if short or long description should be used in subject
            )
        except ObjectDoesNotExist:
            subject = short_description

        # context
        context = {
            'subject': subject,
            'description': description,
            'short_description': short_description,
            'request': request,
            'recipient': recipient,
            'actor': actor,
            'object': object,
            'target': target,
            'details': details,
            'event': event,
            'hash': hash
        }

        if object:
            object_content_type = ContentType.objects.get_for_model(object)
            context[object_content_type.model.lower()] = object

        if target:
            target_content_type = ContentType.objects.get_for_model(target)
            context[target_content_type.model.lower()] = target

        # message
        message = t.render(context)

        # HTML
        if t_html:
            html_message = t_html.render(context)
        else:
            html_message = None

        if whistle_settings.USE_RQ:
            # use background task to release main thread
            from whistle.helpers import send_mail_in_background
            send_mail_in_background.delay(subject, message, settings.DEFAULT_FROM_EMAIL, recipient_list, html_message=html_message, fail_silently=False)
        else:
            # send mail in main thread
            send_mail(subject, message, settings.DEFAULT_FROM_EMAIL, recipient_list, html_message=html_message, fail_silently=False)



from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.core.cache import cache
from django.core.validators import EMPTY_VALUES
from django.db import models
from django.utils.translation import ugettext_lazy as _, get_language, ugettext

from pragmatic.helpers import method_overridden
from whistle.managers import NotificationQuerySet, NotificationManager, EmailManager
from whistle import settings as whistle_settings

try:
    # Python 2
    import urlparse
    from urllib import urlencode
except:
    # Python 3
    import urllib.parse as urlparse
    from urllib.parse import urlencode


class Notification(models.Model):
    recipient = models.ForeignKey(whistle_settings.AUTH_USER_MODEL, on_delete=models.CASCADE, related_name='notifications')
    event = models.CharField(_('event'), choices=whistle_settings.EVENTS, max_length=32, db_index=True)
    actor = models.ForeignKey(whistle_settings.AUTH_USER_MODEL, on_delete=models.SET_NULL,
        blank=True, null=True, default=None)

    object_content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE, related_name='notification_object',
        blank=True, null=True, default=None)
    object_id = models.PositiveIntegerField(
        blank=True, null=True, default=None)
    object = GenericForeignKey(ct_field='object_content_type', fk_field='object_id')

    target_content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE, related_name='notification_target',
        blank=True, null=True, default=None)
    target_id = models.PositiveIntegerField(
        blank=True, null=True, default=None)
    target = GenericForeignKey(ct_field='target_content_type', fk_field='target_id')

    details = models.TextField(_('details'), blank=True, default='')
    is_read = models.BooleanField(_('read'), default=False, db_index=True)
    created = models.DateTimeField(_('created'), auto_now_add=True, db_index=True)
    modified = models.DateTimeField(_('modified'), auto_now=True)
    objects = NotificationQuerySet.as_manager()

    class Meta:
        verbose_name = _('notification')
        verbose_name_plural = _('notifications')
        ordering = ('-created',)

    def __str__(self):
        return self.description

    @property
    def description(self):
        return self.get_description(True)

    @property
    def hash(self):
        from django.core import signing
        protect = {'notification_id': self.pk, 'recipient_id': self.recipient.pk}
        return signing.dumps(protect, key=whistle_settings.SIGNING_KEY, salt=whistle_settings.SIGNING_SALT)

    def short_description(self):
        return self.get_description(False)

    def get_description(self, pass_variables):
        language = get_language()

        cache_key = self.__class__.__name__.lower()
        cache_version = '{}_{}_{}'.format(self.pk, language, pass_variables)
        saved_description = cache.get(cache_key, version=cache_version)

        if saved_description is not None:
            return saved_description

        try:
            description = NotificationManager.get_description(self.event, self.actor, self.object, self.target, pass_variables)
        except KeyError:
            # referenced object does not exist anymore
            if self.pk:
                self.delete()
            return ugettext('Related object does not exist anymore')

        # save into cache
        cache.set(cache_key, description, version=cache_version, timeout=whistle_settings.TIMEOUT)

        return description

    def get_absolute_url(self):
        language = get_language()
        cache_key = 'notification_url'
        cache_version = '{}_{}'.format(self.pk, language)
        saved_url = cache.get(cache_key, version=cache_version)

        if saved_url is not None:
            return saved_url

        try:
            url = '#' if self.object is None else self.object.get_absolute_url()
        except AttributeError:
            url = '#' if self.target is None else self.target.get_absolute_url()

        url_handler = whistle_settings.URL_HANDLER

        if url_handler:
            url = url_handler(url, self)

        if not self.is_read:
            # add read-notification param for middleware
            params = {'read-notification': self.pk}
            url_parts = list(urlparse.urlparse(url))
            query = dict(urlparse.parse_qsl(url_parts[4]))
            query.update(params)
            url_parts[4] = urlencode(query)
            url = urlparse.urlunparse(url_parts)
            
        # save into cache
        cache.set(cache_key, url, version=cache_version, timeout=whistle_settings.TIMEOUT)

        return url

    def send_mail(self, request=None):
        EmailManager.send_mail(
            request=request,
            recipient=self.recipient,
            event=self.event,
            actor=self.actor,
            object=self.object,
            target=self.target,
            details=self.details,
            hash=self.hash
        )

    def push(self, request):
        if not whistle_settings.PUSH_NOTIFICATIONS_ENABLED:
            return

        from fcm_django.models import FCMDevice
        from firebase_admin.messaging import Notification, Message, \
            AndroidConfig, AndroidNotification, APNSPayload, Aps, APNSConfig

        for device in self.recipient.fcmdevice_set.all():
            data = {}
            for data_attr in ['id', 'object_id', 'target_id', 'object_content_type', 'target_content_type']:
                value = getattr(self, data_attr)

                if value:
                    data[data_attr] = '.'.join(value.natural_key()) if isinstance(value, ContentType) else str(value)

            # from objprint import op
            # op(data)

            # TODO: make it more configurable (using custom hander/[lamda] function)
            if self.details not in EMPTY_VALUES:
                title = self.description
                body = self.details
            elif method_overridden(self.object, '__repr__'):
                body = repr(self.object)
                title = self.short_description()
            else:
                body = str(self.object)
                title = self.short_description()

            result = device.send_message(
                Message(
                    notification=Notification(
                        title=title,
                        body=body,
                        # image="image_url"
                    ),
                    data=data,
                    android=AndroidConfig(
                        collapse_key=f'{self.event}_{self.object_id}',
                        priority="high",
                        notification=AndroidNotification(
                            click_action=self.event,
                            sound="default"
                        )
                    ),
                    apns=APNSConfig(
                        payload=APNSPayload(
                            aps=Aps(
                                badge=self.recipient.unread_notifications.count(),
                                category=self.event,
                                sound="default"
                            )
                        )
                    )
                )
            )
            # op(result)



from django.utils.translation import ngettext

from rest_framework import serializers, viewsets
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView

from pragmatic.serializers import ContentTypeSerializer
from whistle.models import Notification


class NotificationSerializer(serializers.ModelSerializer):
    description = serializers.CharField()
    short_description = serializers.CharField()
    object_content_type = ContentTypeSerializer(read_only=True)
    target_content_type = ContentTypeSerializer(read_only=True)

    class Meta:
        model = Notification
        exclude = []


class NotificationViewSet(viewsets.ReadOnlyModelViewSet):
    permission_classes = [IsAuthenticated]
    queryset = Notification.objects.all()
    serializer_class = NotificationSerializer

    def get_queryset(self):
        return super().get_queryset()\
            .for_recipient(self.request.user) \
            .select_related(
                'object_content_type',
                'target_content_type',
                'recipient',
                'actor'
            )


class MarkNotificationsAsReadAPIView(APIView):
    permission_classes = [IsAuthenticated]
    operations = ['apply', 'ignore']

    def patch(self, request):
        notification_id = request.GET.get('notification_id', None)
        unread_notifications = request.user.notifications.unread()

        if notification_id:
            unread_notifications = unread_notifications.filter(id=notification_id)

        num_notifications = unread_notifications.count()
        unread_notifications.mark_as_read()
        request.user.clear_unread_notifications_cache()

        return Response(status=200, data=ngettext(
            '%(count)d notification marked as read',
            '%(count)d notifications marked as read',
            num_notifications,
        ) % {'count': num_notifications})



from django.contrib import admin
from django.contrib.auth import get_user_model
from django.utils.translation import ugettext_lazy as _, ngettext
from whistle.forms import NotificationAdminForm
from whistle.managers import EmailManager
from whistle.models import Notification


@admin.register(Notification)
class NotificationAdmin(admin.ModelAdmin):
    actions = ['make_unread', 'make_read', 'clear_unread_notifications_cache', 'send_email', 'push']
    date_hierarchy = 'created'
    list_select_related = ('recipient', 'actor')
    list_display = ('__str__', 'recipient', 'actor', 'is_read', 'created')
    list_filter = ('event', 'is_read')
    raw_id_fields = ('recipient', 'actor')
    form = NotificationAdminForm

    def make_unread(self, request, queryset):
        rows_updated = queryset.update(is_read=False)

        message = ngettext(
            '%(count)d notification was marked as unread',
            '%(count)d notifications were marked as unread',
            rows_updated
        ) % {
            'count': rows_updated,
        }

        self.clear_unread_notifications_cache(request, queryset)
        self.message_user(request, message)
    make_unread.short_description = _('Make unread')

    def make_read(self, request, queryset):
        rows_updated = queryset.update(is_read=True)

        message = ngettext(
            '%(count)d notification was marked as read',
            '%(count)d notifications were marked as read',
            rows_updated
        ) % {
            'count': rows_updated,
        }

        self.clear_unread_notifications_cache(request, queryset)
        self.message_user(request, message)
    make_read.short_description = _('Make read')

    def clear_unread_notifications_cache(self, request, queryset):
        recipients = get_user_model().objects.filter(pk__in=queryset.values_list('recipient', flat=True))

        if recipients.exists():
            for recipient in recipients:
                recipient.clear_unread_notifications_cache()
            self.message_user(request, _('Unread notifications cache cleared'))
    clear_unread_notifications_cache.short_description = _('Clear unread notifications cache')

    def send_email(self, request, queryset):
        for notification in queryset:
            notification.send_mail(request)
    send_email.short_description = _('Send email')

    def push(self, request, queryset):
        for notification in queryset:
            notification.push(request)
    push.short_description = _('Push')



# Generated by Django 2.0.3 on 2018-03-08 15:13

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False)),
                ('event', models.CharField(db_index=True, max_length=32)),
                ('object_id', models.PositiveIntegerField(blank=True, default=None, null=True)),
                ('target_id', models.PositiveIntegerField(blank=True, default=None, null=True)),
                ('is_read', models.BooleanField(db_index=True, default=False)),
                ('created', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('actor', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
                ('object_content_type', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='notification_object', to='contenttypes.ContentType')),
                ('recipient', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='notifications', to=settings.AUTH_USER_MODEL)),
                ('target_content_type', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='notification_target', to='contenttypes.ContentType')),
            ],
            options={
                'verbose_name': 'notification',
                'ordering': ('-created',),
                'verbose_name_plural': 'notifications',
            },
        ),
    ]



#!/usr/bin/env python
from setuptools import setup

from whistle import VERSION

setup(
    name='django-whistle',
    version=VERSION,
    description='Advanced notifications for Django',
    long_description=open('README.md').read(),
    author='Pragmatic Mates',
    author_email='info@pragmaticmates.com',
    maintainer='Pragmatic Mates',
    maintainer_email='info@pragmaticmates.com',
    url='https://github.com/PragmaticMates/django-whistle',
    packages=[
        'whistle',
        'whistle.migrations'
    ],
    include_package_data=True,
    install_requires=('django', 'django_rq', 'django-crispy-forms', 'django-pragmatic>=4.1.0'),
    classifiers=[
        'Programming Language :: Python',
        'Operating System :: OS Independent',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Framework :: Django',
        'License :: OSI Approved :: BSD License',
        'Development Status :: 5 - Production/Stable'
    ],
    license='BSD License',
    keywords="django notifications events push email iOS Android APN FireBase GCM",
)



try:
    # Django 3.1
    from django.db.models import JSONField
except ImportError:
    # older Django
    from django.contrib.postgres.fields import JSONField

from django.core.cache import cache
from django.db import models
from whistle import settings as whistle_settings


class UserNotificationsMixin(models.Model):
    notification_settings = JSONField(blank=True, null=True, default=None)

    class Meta:
        abstract = True

    @property
    def unread_notifications(self):
        cache_key = 'user_unread_notifications'
        cache_version = self.pk
        saved_notifications = cache.get(cache_key, version=cache_version)

        if saved_notifications is not None:
            return saved_notifications

        unread_notifications = self.notifications.unread().select_related('actor')

        for notification in unread_notifications:
            if notification.object:
                notification.object_display = str(notification.object)

                try:
                    notification.object_url = notification.object.get_absolute_url()
                except AttributeError:
                    notification.object_url = '#'

                notification.object_model = notification.object.__class__.__name__

            if notification.target:
                notification.target_display = str(notification.target)

                try:
                    notification.target_url = notification.target.get_absolute_url()
                except AttributeError:
                    notification.target_url = '#'

                notification.target_model = notification.target.__class__.__name__

        # save into cache
        cache.set(cache_key, unread_notifications, version=cache_version, timeout=whistle_settings.TIMEOUT)

        return unread_notifications

    def clear_unread_notifications_cache(self):
        cache_key = 'user_unread_notifications'
        cache.delete(cache_key, version=self.pk)



# Generated by Django 2.0.6 on 2019-02-19 21:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('whistle', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='notification',
            name='details',
            field=models.TextField(blank=True, default=''),
        ),
    ]



from django.conf.urls import url
from django.utils.translation import pgettext_lazy
from whistle.views import NotificationListView, NotificationSettingsView, ReadNotificationByHashView

app_name = 'notifications'

urlpatterns = [
    url(pgettext_lazy("url", r'^settings/$'), NotificationSettingsView.as_view(), name='settings'),
    url(pgettext_lazy("url", r'^read-notification/(?P<hash>[-\w:]+)/$'), ReadNotificationByHashView.as_view(), name='read_notification'),
    url(r'^$', NotificationListView.as_view(), name='list'),
]



from django.conf import settings
from django.core.cache.backends.base import DEFAULT_TIMEOUT

EVENTS = getattr(settings, 'WHISTLE_NOTIFICATION_EVENTS', [])
URL_HANDLER = getattr(settings, 'WHISTLE_URL_HANDLER', None)
TIMEOUT = getattr(settings, 'WHISTLE_CACHE_TIMEOUT', DEFAULT_TIMEOUT)
USE_RQ = getattr(settings, 'WHISTLE_USE_RQ', True)
REDIS_QUEUE = getattr(settings, 'WHISTLE_REDIS_QUEUE', 'default')
SIGNING_KEY = getattr(settings, 'WHISTLE_SIGNING_KEY', settings.SECRET_KEY)
SIGNING_SALT = getattr(settings, 'WHISTLE_SIGNING_SALT', 'whistle')
AUTH_USER_MODEL = getattr(settings, 'WHISTLE_AUTH_USER_MODEL', settings.AUTH_USER_MODEL)
PUSH_NOTIFICATIONS_ENABLED = 'fcm_django' in settings.INSTALLED_APPS
